@@ -46,6 +46,10 @@ class CannotFindClonedTemplateException(FlexlateTemplateException):
     pass
 
 
+class TemplateSourceWithNameAlreadyExistsException(FlexlateTemplateException):
+    pass
+
+
 class FlexlateGitException(FlexlateException):
     pass
 
@@ -58,6 +62,10 @@ class GitRepoHasNoCommitsException(FlexlateGitException):
     pass
 
 
+class TriedToCommitButNoChangesException(FlexlateGitException):
+    pass
+
+
 class FlexlateConfigException(FlexlateException):
     pass
 
@@ -130,3 +138,11 @@ class MergeCommitIsNotMergingAFlexlateTransactionException(
 
 class CannotFindMergeForTransactionException(FlexlateTransactionException):
     pass
+
+
+class FlexlateSyncException(FlexlateException):
+    pass
+
+
+class UnnecessarySyncException(FlexlateSyncException):
+    pass
