@@ -1,3 +1,4 @@
+import json
 import os
 from dataclasses import dataclass
 from unittest.mock import patch
@@ -6,10 +7,13 @@ import appdirs
 import pytest
 from git import Head
 
+from flexlate import branch_update
 from flexlate.add_mode import AddMode
 from flexlate.config import FlexlateConfig, FlexlateProjectConfig
 from flexlate.constants import DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME
 from flexlate.exc import TemplateSourceWithNameAlreadyExistsException
+from flexlate.ext_git import repo_has_merge_conflicts
+from flexlate.template.base import Template
 from flexlate.template.copier import CopierTemplate
 from flexlate.template.types import TemplateType
 from flexlate.transactions.transaction import FlexlateTransaction
@@ -21,6 +25,10 @@ from tests.fixtures.template import *
 from tests.fixtures.templated_repo import *
 from tests.fixtures.add_mode import add_mode
 from tests.fixtures.transaction import *
+from tests.gitutils import (
+    accept_theirs_in_merge_conflict,
+    assert_main_commit_message_matches,
+)
 
 
 def test_add_template_source_to_repo(
@@ -37,11 +45,20 @@ def test_add_template_source_to_repo(
         out_root=GENERATED_REPO_DIR,
         target_version="some version",
     )
+    _assert_template_source_cookiecutter_one_added_correctly(cookiecutter_one_template)
+
+
+def _assert_template_source_cookiecutter_one_added_correctly(
+    cookiecutter_one_template: CookiecutterTemplate,
+    num_sources: int = 1,
+    source_idx: int = 0,
+    num_applied_templates: int = 0,
+):
     config_path = GENERATED_REPO_DIR / "flexlate.json"
     config = FlexlateConfig.load(config_path)
-    assert len(config.applied_templates) == 0
-    assert len(config.template_sources) == 1
-    source = config.template_sources[0]
+    assert len(config.applied_templates) == num_applied_templates
+    assert len(config.template_sources) == num_sources
+    source = config.template_sources[source_idx]
     assert source.name == cookiecutter_one_template.name
     assert source.path == str(cookiecutter_one_template.path)
     assert source.version == cookiecutter_one_template.version
@@ -186,15 +203,9 @@ def test_add_remote_cookiecutter_applied_template_to_repo(
     else:
         raise ValueError(f"unsupported add mode {add_mode}")
 
-    config_path = config_dir / "flexlate.json"
-    config = FlexlateConfig.load(config_path)
-    assert len(config.applied_templates) == 1
-    at = config.applied_templates[0]
-    assert at.name == template.name
-    assert at.version == template.version
-    assert at.data == {"name": "abc", "key": "value"}
-    assert at.root == template_root
-    assert at.add_mode == add_mode
+    _assert_remote_cookiecutter_applied_correctly(
+        template, template_root, add_mode, config_dir=config_dir
+    )
 
     template_sources_config_path = GENERATED_REPO_DIR / "flexlate.json"
     ts_config = FlexlateConfig.load(template_sources_config_path)
@@ -208,6 +219,23 @@ def test_add_remote_cookiecutter_applied_template_to_repo(
     assert source.render_relative_root_in_template == Path("{{ cookiecutter.name }}")
 
 
+def _assert_remote_cookiecutter_applied_correctly(
+    template: Template,
+    template_root: Path,
+    add_mode: AddMode,
+    config_dir: Path = GENERATED_REPO_DIR,
+):
+    config_path = config_dir / "flexlate.json"
+    config = FlexlateConfig.load(config_path)
+    assert len(config.applied_templates) == 1
+    at = config.applied_templates[0]
+    assert at.name == template.name
+    assert at.version == template.version
+    assert at.data == {"name": "abc", "key": "value"}
+    assert at.root == template_root
+    assert at.add_mode == add_mode
+
+
 def test_add_source_and_output_at_target_version(
     repo_with_placeholder_committed: Repo,
     cookiecutter_remote_version_one_template: CookiecutterTemplate,
@@ -442,7 +470,102 @@ def test_add_source_to_project_with_existing_outputs(
     assert len(at_config.applied_templates) == 1
 
 
-# TODO: test for adding to existing
+def test_add_source_with_merge_conflicts_and_resolution(
+    repo_with_cookiecutter_remote_version_one_template_source_and_output_that_will_have_merge_conflict_on_flexlate_operation,
+    cookiecutter_one_template: CookiecutterTemplate,
+    add_source_transaction: FlexlateTransaction,
+):
+    repo = repo_with_cookiecutter_remote_version_one_template_source_and_output_that_will_have_merge_conflict_on_flexlate_operation
+
+    adder = Adder()
+
+    def _resolve_conflicts_then_type_yes(prompt: str) -> bool:
+        assert repo_has_merge_conflicts(repo)
+        accept_theirs_in_merge_conflict(repo)
+        stage_and_commit_all(repo, "Manually resolve conflicts")
+        return True
+
+    with patch.object(branch_update, "confirm_user", _resolve_conflicts_then_type_yes):
+        adder.add_template_source(
+            repo,
+            cookiecutter_one_template,
+            add_source_transaction,
+            out_root=GENERATED_REPO_DIR,
+            target_version="some version",
+        )
+
+    _assert_template_source_cookiecutter_one_added_correctly(
+        cookiecutter_one_template, num_sources=2, source_idx=1, num_applied_templates=1
+    )
+
+
+def test_add_source_with_merge_conflicts_and_abort(
+    repo_with_cookiecutter_remote_version_one_template_source_and_output_that_will_have_merge_conflict_on_flexlate_operation,
+    cookiecutter_one_template: CookiecutterTemplate,
+    add_source_transaction: FlexlateTransaction,
+):
+    repo = repo_with_cookiecutter_remote_version_one_template_source_and_output_that_will_have_merge_conflict_on_flexlate_operation
+
+    adder = Adder()
+
+    def _reject(prompt: str) -> bool:
+        return False
+
+    with patch.object(branch_update, "confirm_user", _reject):
+        adder.add_template_source(
+            repo,
+            cookiecutter_one_template,
+            add_source_transaction,
+            out_root=GENERATED_REPO_DIR,
+            target_version="some version",
+        )
+
+    config_path = GENERATED_REPO_DIR / "flexlate.json"
+    config = FlexlateConfig.load(config_path)
+    assert len(config.template_sources) == 1
+
+    assert_main_commit_message_matches(
+        repo.commit().message, "Reformat flexlate config"
+    )
+    for branch_name in [DEFAULT_MERGED_BRANCH_NAME, DEFAULT_TEMPLATE_BRANCH_NAME]:
+        branch = repo.branches[branch_name]  # type: ignore
+        branch.checkout()
+        assert_main_commit_message_matches(
+            repo.commit().message, "Update flexlate templates"
+        )
+
+
+def test_add_output_with_merge_conflicts_and_resolution(
+    repo_with_cookiecutter_remote_version_one_template_source_and_output_that_will_have_merge_conflict_on_flexlate_operation: Repo,
+    cookiecutter_remote_template: CookiecutterTemplate,
+    add_output_transaction: FlexlateTransaction,
+):
+    repo = repo_with_cookiecutter_remote_version_one_template_source_and_output_that_will_have_merge_conflict_on_flexlate_operation
+    template = cookiecutter_remote_template
+
+    adder = Adder()
+
+    def _resolve_conflicts_then_type_yes(prompt: str) -> bool:
+        assert repo_has_merge_conflicts(repo)
+        accept_theirs_in_merge_conflict(repo)
+        stage_and_commit_all(repo, "Manually resolve conflicts")
+        return True
+
+    subdir = GENERATED_REPO_DIR / "subdir"
+    subdir.mkdir()
+
+    with patch.object(branch_update, "confirm_user", _resolve_conflicts_then_type_yes):
+        with change_directory_to(subdir):
+            adder.apply_template_and_add(
+                repo,
+                template,
+                add_output_transaction,
+                no_input=True,
+            )
+
+    _assert_remote_cookiecutter_applied_correctly(
+        template, Path(".."), AddMode.LOCAL, config_dir=subdir / "abc"
+    )
 
 
 def test_add_project_config_with_git(repo_with_placeholder_committed: Repo):
